import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class CSpaceVisual extends JComponent{
	
	private JFrame		frame;
	private ArmVisual	arms[];
	
	private Obj			objects[];
	
	private boolean		space[][];
	
	public CSpaceVisual(Arm arm[],Robot bot){
		frame=new JFrame("Configuration Space Visualizer");
		frame.setLayout(null);
		
		frame.setSize(220+(arm.length*20),250);
		
		Container pane=frame.getContentPane();
		pane.add(this);
		
		arms=new ArmVisual[arm.length];
		
		for(int i=0; i<arm.length; i++){
			pane.add((arms[i]=new ArmVisual(arm[i])));
			arms[i].setLocation(200+(i*20),0);
			arms[i].setSize(arms[i].getPreferredSize());
		}
		
		setSize(200,200);
		setLocation(0,0);
		frame.setLocation(500,0);
		frame.setBackground(Color.WHITE);
		pane.setBackground(Color.WHITE);
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		frame.setVisible(true);
		
		space=new boolean[360][360];
		
		Point2D.Double pnt1,pnt2;
		objects=new Obj[2];
		objects[0]=new Obj(new Rectangle(-20,30,40,70));
		objects[1]=new Obj(new Rectangle(-100,-51,200,50));
		
		bot.setObjects(objects);
		
		for(int x=0; x<360; x++){
			for(int y=0; y<360; y++){
				
				arms[0].set(x);
				arms[1].set(y);
				
				pnt1=arm[0].getPoint();
				pnt2=arm[1].getPoint();
				
				space[x][y] = false;
				
				for(int i=0; i<objects.length; i++){
					if((objects[i].collide(pnt1)||objects[i].collide(pnt2))){
						space[x][y] = true;
					}
				}
				//				space[x][y]=(objects[0].collide(pnt1)||objects[0].collide(pnt2));
			}
		}
		
		addMouseMotionListener(new MouseMotionListener(){
			
			@Override
			public void mouseDragged(MouseEvent e){
				int x,y,xsize,ysize;
				Dimension size;
				double xs,ys;
				
				size=getSize();
				xsize=size.width;
				ysize=size.height;
				
				xs=(double) 360/(double) xsize;
				ys=(double) 360/(double) ysize;
				
				x=e.getX();
				y=e.getY();
				
				if(x>xsize) x=xsize;
				if(x<0) x=0;
				if(y>ysize) y=ysize;
				if(y<0) y=0;
				
				x=(int) (x*xs);
				y=(int) (y*ys);
				
				arms[0].set(x);
				arms[1].set(y);
			}
			
			@Override
			public void mouseMoved(MouseEvent e){}
			
		});
		
		repaint();
	}
	
	@Override
	public void paint(Graphics g){
		double xs,ys;
		xs=(double) 360/(double) 200;
		ys=(double) 360/(double) 200;
		
		for(int y=0; y<360; y++){
			for(int x=0; x<360; x++){
				if(space[x][y]) g.drawLine((int) (x/xs),(int) (y/ys),(int) (x/xs),(int) (y/ys));
				//				System.out.print(space[x][y]?1 : 0);
			}
		}
	}
	
	private class ArmVisual extends JSlider implements ChangeListener{
		
		private Arm	arm;
		
		public ArmVisual(Arm arm){
			super(VERTICAL,0,360,180);
			Dimension d=new Dimension(20,200);
			setMinimumSize(d);
			setPreferredSize(d);
			setMaximumSize(d);
			
			addChangeListener(this);
			
			this.arm=arm;
		}
		
		public void set(int i){
			setValue(i);
		}
		
		public int get(){
			return getValue();
		}
		
		@Override
		public void stateChanged(ChangeEvent arg0){
			arm.setDegree(get());
		}
	}
}
